@@ -1,5 +1,10 @@
 """
 For lazy man
+
+.. versionadded:: v2.15.0
+	Module ``mcdreforged`` now contains everything in ``mcdreforged.api.all``. All MCDR API components can be imported from here
+
+	Now it's suggested to import all MCDR API components directly from the ``mcdreforged`` module
 """
 
 # noinspection PyUnresolvedReferences
