@@ -1,4 +1,5 @@
-from mcdreforged.command.builder import command_builder_util
+from mcdreforged.command.builder import command_builder_utils
+from mcdreforged.command.builder import command_builder_utils as command_builder_util  # backwards compatibility
 from mcdreforged.command.builder.common import CommandContext, ParseResult
 from mcdreforged.command.builder.exception import LiteralNotMatch, NumberOutOfRange, IllegalArgument, EmptyText, \
 	UnknownCommand, UnknownArgument, CommandSyntaxError, UnknownRootArgument, RequirementNotMet, IllegalNodeOperation, \
@@ -7,19 +8,31 @@ from mcdreforged.command.builder.exception import LiteralNotMatch, NumberOutOfRa
 from mcdreforged.command.builder.nodes.arguments import Number, Integer, Float, Text, QuotableText, \
 	GreedyText, Boolean, Enumeration
 from mcdreforged.command.builder.nodes.basic import AbstractNode, Literal, ArgumentNode
+from mcdreforged.command.builder.nodes.special import CountingLiteral
 from mcdreforged.command.builder.tools import SimpleCommandBuilder, Requirements, NodeDefinition
 
 __all__ = [
 	# ------------------
-	#   Argument Nodes
+	#   Basic Nodes
 	# ------------------
 
 	'AbstractNode', 'ArgumentNode',
 	'Literal',
+
+	# ------------------
+	#   Argument Nodes
+	# ------------------
+
 	'Number', 'Integer', 'Float',
 	'Text', 'QuotableText', 'GreedyText',
 	'Boolean', 'Enumeration',
 
+	# ------------------
+	#   Special Nodes
+	# ------------------
+
+	'CountingLiteral',
+
 	# ------------------
 	#     Exceptions
 	# ------------------
@@ -42,7 +55,7 @@ __all__ = [
 	# ------------------
 
 	'CommandContext',
-	'command_builder_util',
+	'command_builder_util', 'command_builder_utils',
 	'ParseResult',
 
 	# ------------------
