@@ -1,11 +1,15 @@
+import dataclasses
 import typing
 from contextlib import contextmanager
 from typing import List, Iterable, Dict, Any, Optional, NamedTuple
 
-from mcdreforged.command.command_source import CommandSource
+from typing_extensions import override
+
+from mcdreforged.command.builder.callback import ScheduledCallback
 
 if typing.TYPE_CHECKING:
-	from mcdreforged.command.builder.nodes.basic import AbstractNode, ArgumentNode
+	from mcdreforged.command.command_source import CommandSource
+	from mcdreforged.command.builder.nodes.basic import AbstractNode
 
 
 class ParseResult(NamedTuple):
@@ -13,6 +17,16 @@ class ParseResult(NamedTuple):
 	char_read: int
 
 
+@dataclasses.dataclass(frozen=True)
+class CommandExecution:
+	context: 'CommandContext'
+	scheduled_callback: ScheduledCallback
+
+
+class CommandExecutions(List[CommandExecution]):
+	pass
+
+
 class CommandSuggestion:
 	def __init__(self, command_read: str, suggest_segment: str):
 		self.__command_read = command_read
@@ -50,6 +64,7 @@ class CommandSuggestions(List[CommandSuggestion]):
 		# "<file_name>" is the complete_hint
 		self.complete_hint: Optional[str] = None
 
+	@override
 	def extend(self, __iterable: Iterable) -> None:
 		super().extend(__iterable)
 		if isinstance(__iterable, CommandSuggestions):
@@ -72,8 +87,9 @@ class CommandContext(Dict[str, Any]):
 		self.__source = source
 		self.__command = command
 		self.__cursor = 0
-		self.__node_path = []  # type: List[AbstractNode]
+		self.__node_path: List[AbstractNode] = []
 
+	@override
 	def copy(self) -> 'CommandContext':
 		copied = CommandContext(self.source, self.command)
 		copied.update(self)
@@ -128,26 +144,25 @@ class CommandContext(Dict[str, Any]):
 	# -------------------------
 
 	@contextmanager
-	def read_command(self, current_node: 'AbstractNode', result: 'ParseResult', new_cursor: int):
+	def visit_node(self, current_node: 'AbstractNode', result: 'ParseResult', new_cursor: int):
 		"""
 		**Not public API, only used in command parsing**
 		Change the current cursor position, and store the parsing value
 
 		:meta private:
 		"""
-		from mcdreforged.command.builder.nodes.basic import ArgumentNode
-
 		prev_cursor = self.__cursor
-		self.__cursor = new_cursor
+		prev_data = dict(self)
 
-		if isinstance(current_node, ArgumentNode):
-			self[current_node.get_name()] = result.value
+		self.__cursor = new_cursor
 		try:
+			# noinspection PyProtectedMember
+			current_node._on_visited(self, result)
 			yield
 		finally:
 			self.__cursor = prev_cursor
-			if isinstance(current_node, ArgumentNode):
-				self.pop(current_node.get_name(), None)
+			self.clear()
+			self.update(prev_data)
 
 	@contextmanager
 	def enter_child(self, node: 'AbstractNode'):
