@@ -1,8 +1,10 @@
 from abc import ABC
 from typing import Optional, Union
 
+from typing_extensions import override
+
 from mcdreforged.minecraft.rtext.text import RTextBase, RText, RColor
-from mcdreforged.utils.types import MessageText
+from mcdreforged.utils.types.message import MessageText
 
 
 class CommandErrorBase(Exception, ABC):
@@ -75,7 +77,7 @@ class CommandError(CommandErrorBase, ABC):
 		"""
 		return ()
 
-	def set_message(self, message: str):
+	def set_message(self, message: MessageText):
 		self.__message = message
 
 	def get_parsed_command(self) -> str:
@@ -152,6 +154,7 @@ class RequirementNotMet(CommandError):
 	def get_reason(self) -> MessageText:
 		return self.__reason
 
+	@override
 	def get_error_data(self) -> tuple:
 		return (self.get_reason(),)
 
@@ -164,10 +167,10 @@ class CommandSyntaxError(CommandError, ABC):
 	"""
 	The basic exception for command parsing error
 	"""
-	def __init__(self, message: str, char_read: Union[int, str]):
-		super().__init__(message, '', '?' if isinstance(char_read, int) else char_read)
-		self.message = message
-		self.char_read = char_read if isinstance(char_read, int) else len(char_read)
+	def __init__(self, message: MessageText, char_read: Union[int, str]):
+		# parsed_command and failed_command will be set in mcdreforged.command.builder.nodes.basic.AbstractNode._execute_command
+		super().__init__(message, '', '' if isinstance(char_read, int) else char_read)
+		self.char_read: int = char_read if isinstance(char_read, int) else len(char_read)
 
 	def set_parsed_command(self, parsed_command):
 		self._parsed_command = parsed_command
@@ -198,7 +201,7 @@ class AbstractOutOfRange(IllegalArgument, ABC):
 	"""
 	The basic exception for out-of-range like argument parsing error
 	"""
-	def __init__(self, message: str, char_read: Union[int, str], value, range_l, range_r):
+	def __init__(self, message: MessageText, char_read: Union[int, str], value, range_l, range_r):
 		"""
 		:param value: The actual value
 		:param range_l: The left boundary
@@ -213,6 +216,7 @@ class AbstractOutOfRange(IllegalArgument, ABC):
 	def get_boundary_text(cls, value) -> str:
 		return str(value) if value is not None else '/'
 
+	@override
 	def get_error_data(self) -> tuple:
 		return self.__value, self.get_boundary_text(self.__range_l), self.get_boundary_text(self.__range_r)
 
