@@ -3,18 +3,22 @@ from abc import ABC
 from enum import Enum
 from typing import Type, Iterable, Union, Optional
 
-from mcdreforged.command.builder import command_builder_util as utils
-from mcdreforged.command.builder.command_builder_util import DIVIDER
+from typing_extensions import override, Unpack
+
+from mcdreforged.command.builder import command_builder_utils as utils
+from mcdreforged.command.builder.command_builder_utils import DIVIDER
 from mcdreforged.command.builder.common import ParseResult, CommandContext
 from mcdreforged.command.builder.exception import NumberOutOfRange, EmptyText, \
 	InvalidNumber, InvalidInteger, InvalidFloat, UnclosedQuotedString, IllegalEscapesUsage, \
 	TextLengthOutOfRange, InvalidBoolean, InvalidEnumeration, AbstractOutOfRange
 from mcdreforged.command.builder.nodes.basic import SUGGESTS_CALLBACK, \
 	ArgumentNode
+from mcdreforged.utils import misc_utils, class_utils
+
+
 # --------------------
 #   Number Arguments
 # --------------------
-from mcdreforged.utils import misc_util
 
 
 class NumberNode(ArgumentNode, ABC):
@@ -32,8 +36,8 @@ class NumberNode(ArgumentNode, ABC):
 
 	NumberType = Union[int, float]
 
-	def __init__(self, name):
-		super().__init__(name)
+	def __init__(self, name, **kwargs: Unpack[ArgumentNode._InitKwargs]):
+		super().__init__(name, **kwargs)
 		self.__min_value: Optional[NumberNode.NumberType] = None
 		self.__max_value: Optional[NumberNode.NumberType] = None
 
@@ -85,7 +89,11 @@ class NumberNode(ArgumentNode, ABC):
 		return super().__str__() + extra
 
 	def __repr__(self):
-		return '{}[name={},min={},max={}]'.format(self.__class__.__name__, self.get_name(), self.__min_value, self.__max_value)
+		return class_utils.represent(self, {
+			'name': self.__name,
+			'min': self.__min_value,
+			'max': self.__max_value,
+		})
 
 
 class Number(NumberNode):
@@ -95,7 +103,9 @@ class Number(NumberNode):
 	If the next element is not a number,
 	a :class:`~mcdreforged.command.builder.exception.InvalidNumber` exception will be risen
 	"""
+	@override
 	def parse(self, text: str) -> ParseResult:
+		value: Optional[NumberNode.NumberType]
 		value, read = utils.get_int(text)
 		if value is None:
 			value, read = utils.get_float(text)
@@ -112,6 +122,7 @@ class Integer(NumberNode):
 	If the next element is not an integer,
 	a :class:`~mcdreforged.command.builder.exception.InvalidInteger` exception will be risen
 	"""
+	@override
 	def parse(self, text: str) -> ParseResult:
 		value, read = utils.get_int(text)
 		if value is not None:
@@ -127,6 +138,7 @@ class Float(NumberNode):
 	If the next element is not a float,
 	a :class:`~mcdreforged.command.builder.exception.InvalidFloat` exception will be risen
 	"""
+	@override
 	def parse(self, text: str) -> ParseResult:
 		value, read = utils.get_float(text)
 		if value is not None:
@@ -150,10 +162,10 @@ class TextNode(ArgumentNode, ABC):
 
 	By default, there's no length range restriction
 	"""
-	def __init__(self, name):
-		super().__init__(name)
-		self.__min_length = None
-		self.__max_length = None
+	def __init__(self, name: str, **kwargs: Unpack[ArgumentNode._InitKwargs]):
+		super().__init__(name, **kwargs)
+		self.__min_length: Optional[int] = None
+		self.__max_length: Optional[int] = None
 
 	def at_min_length(self, min_length: int) -> 'TextNode':
 		"""
@@ -204,7 +216,11 @@ class TextNode(ArgumentNode, ABC):
 		return super().__str__() + extra
 
 	def __repr__(self):
-		return '{}[name={},min_len={},max_len={}]'.format(self.__class__.__name__, self.get_name(), self.__min_length, self.__max_length)
+		return class_utils.represent(self, {
+			'name': self.__name,
+			'min_len': self.__min_length,
+			'max_len': self.__max_length,
+		})
 
 
 class Text(TextNode):
@@ -213,6 +229,7 @@ class Text(TextNode):
 
 	It will keep reading chars continuously until it meets a space character
 	"""
+	@override
 	def parse(self, text: str) -> ParseResult:
 		arg = utils.get_element(text)
 		return self._check_length_in_range_and_return(arg, len(arg))
@@ -237,14 +254,15 @@ class QuotableText(Text):
 	QUOTE_CHAR = '"'
 	ESCAPE_CHAR = '\\'
 
-	def __init__(self, name):
-		super().__init__(name)
+	def __init__(self, name, **kwargs: Unpack[ArgumentNode._InitKwargs]):
+		super().__init__(name, **kwargs)
 		self.empty_allowed = False
 
 	def allow_empty(self):
 		self.empty_allowed = True
 		return self
 
+	@override
 	def parse(self, text: str) -> ParseResult:
 		if len(text) == 0 or text[0] != self.QUOTE_CHAR:
 			return super().parse(text)  # regular text
@@ -272,6 +290,7 @@ class QuotableText(Text):
 		raise UnclosedQuotedString(text)
 
 	# use quote characters to quote suggestions with DIVIDER
+	@override
 	def suggests(self, suggestion: SUGGESTS_CALLBACK) -> 'QuotableText':
 		def quote_wrapper(*args, **kwargs):
 			suggestions = []
@@ -282,7 +301,7 @@ class QuotableText(Text):
 			return suggestions
 
 		# noinspection PyTypeChecker
-		return super().suggests(misc_util.copy_signature(quote_wrapper, suggestion))
+		return super().suggests(misc_utils.copy_signature(quote_wrapper, suggestion))
 
 
 class GreedyText(TextNode):
@@ -293,6 +312,7 @@ class GreedyText(TextNode):
 
 	It's not a smart decision to append any child nodes to a :class:`GreedyText`, since the child nodes can never get any remaining command
 	"""
+	@override
 	def parse(self, text: str) -> ParseResult:
 		return self._check_length_in_range_and_return(text, len(text))
 
@@ -309,9 +329,11 @@ class Boolean(ArgumentNode):
 
 	.. versionadded:: v2.3.0
 	"""
+	@override
 	def _get_suggestions(self, context: CommandContext) -> Iterable[str]:
 		return ['true', 'false']
 
+	@override
 	def parse(self, text: str) -> ParseResult:
 		arg = utils.get_element(text)
 		if arg.lower() == 'true':
@@ -343,13 +365,15 @@ class Enumeration(ArgumentNode):
 
 	.. versionadded:: v2.3.0
 	"""
-	def __init__(self, name: str, enum_class: Type[Enum]):
-		super().__init__(name)
+	def __init__(self, name: str, enum_class: Type[Enum], **kwargs: Unpack[ArgumentNode._InitKwargs]):
+		super().__init__(name, **kwargs)
 		self.__enum_class: Type[Enum] = enum_class
 
+	@override
 	def _get_suggestions(self, context: CommandContext) -> Iterable[str]:
-		return map(lambda e: e.name, self.__enum_class)
+		return [e.name for e in self.__enum_class]
 
+	@override
 	def parse(self, text: str) -> ParseResult:
 		arg = utils.get_element(text)
 		try:
@@ -360,8 +384,11 @@ class Enumeration(ArgumentNode):
 			return ParseResult(enum, len(arg))
 
 	def __str__(self):
-		return super().__str__() + ' @ {}'.format(self.__class__.__name__, self.get_name(), self.__enum_class.__name__)
+		return super().__str__() + ' ({})'.format(self.__enum_class.__name__)
 
 	def __repr__(self):
-		return '{}[name={},enum_class={}]'.format(self.__class__.__name__, self.get_name(), self.__enum_class)
+		return class_utils.represent(self, {
+			'name': self.__name,
+			'enum_class': self.__enum_class,
+		})
 
