@@ -1,43 +1,69 @@
 import collections
-import inspect
-from abc import ABC
-from types import MethodType
-from typing import List, Callable, Iterable, Set, Dict, Type, Any, Union, Optional, NamedTuple, TypeVar
+import dataclasses
+from abc import ABC, abstractmethod
+from typing import List, Callable, Iterable, Set, Dict, Type, Any, Union, Optional, TypedDict, TypeVar, NoReturn
 
-from mcdreforged.command.builder import command_builder_util as utils
-from mcdreforged.command.builder.common import ParseResult, CommandContext, CommandSuggestions, CommandSuggestion
+from typing_extensions import Self, override, NotRequired
+
+from mcdreforged.command.builder import command_builder_utils as utils
+from mcdreforged.command.builder.callback import CallbackError, ScheduledCallback, DirectCallbackInvoker
+from mcdreforged.command.builder.common import ParseResult, CommandContext, CommandSuggestions, CommandSuggestion, CommandExecutions, CommandExecution
 from mcdreforged.command.builder.exception import LiteralNotMatch, UnknownCommand, UnknownArgument, CommandSyntaxError, \
 	UnknownRootArgument, RequirementNotMet, IllegalNodeOperation, \
 	CommandError
 from mcdreforged.command.command_source import CommandSource
-from mcdreforged.utils import misc_util, tree_printer, class_util
-from mcdreforged.utils.types import MessageText
+from mcdreforged.utils import tree_printer, class_utils, collection_utils
+from mcdreforged.utils.types.message import MessageText
 
-__SOURCE_CONTEXT_CALLBACK = Union[Callable[[], Any], Callable[[CommandSource], Any], Callable[[CommandSource, dict], Any]]
-__SOURCE_CONTEXT_CALLBACK_BOOL = Union[Callable[[], bool], Callable[[CommandSource], bool], Callable[[CommandSource, dict], bool]]
-__SOURCE_CONTEXT_CALLBACK_MSG = Union[Callable[[], MessageText], Callable[[CommandSource], MessageText], Callable[[CommandSource, dict], MessageText]]
-__SOURCE_CONTEXT_CALLBACK_STR_ITERABLE = Union[Callable[[], Iterable[str]], Callable[[CommandSource], Iterable[str]], Callable[[CommandSource, dict], Iterable[str]]]
-__SOURCE_ERROR_CONTEXT_CALLBACK = Union[Callable[[], Any], Callable[[CommandSource], Any], Callable[[CommandSource, CommandError], Any], Callable[[CommandSource, CommandError, dict], Any]]
+_T = TypeVar('_T')
+__SOURCE_CONTEXT_CALLBACK = Union[
+	Callable[[], Any],
+	Callable[[CommandSource], Any],
+	Callable[[CommandSource, CommandContext], Any]
+]
+__SOURCE_CONTEXT_CALLBACK_BOOL = Union[
+	Callable[[], bool],
+	Callable[[CommandSource], bool],
+	Callable[[CommandSource, CommandContext], bool]
+]
+__SOURCE_CONTEXT_CALLBACK_MSG = Union[
+	Callable[[], MessageText],
+	Callable[[CommandSource], MessageText],
+	Callable[[CommandSource, CommandContext], MessageText]
+]
+__SOURCE_CONTEXT_CALLBACK_STR_ITERABLE = Union[
+	Callable[[], Iterable[str]],
+	Callable[[CommandSource], Iterable[str]],
+	Callable[[CommandSource, CommandContext], Iterable[str]]
+]
+__SOURCE_ERROR_CONTEXT_CALLBACK = Union[
+	Callable[[], Any],
+	Callable[[CommandSource], Any],
+	Callable[[CommandSource, CommandError], Any],
+	Callable[[CommandSource, CommandError, CommandContext], Any]
+]
 
 RUNS_CALLBACK = __SOURCE_CONTEXT_CALLBACK
 ERROR_HANDLER_CALLBACK = __SOURCE_ERROR_CONTEXT_CALLBACK
 FAIL_MSG_CALLBACK = __SOURCE_CONTEXT_CALLBACK_MSG
 SUGGESTS_CALLBACK = __SOURCE_CONTEXT_CALLBACK_STR_ITERABLE
 REQUIRES_CALLBACK = __SOURCE_CONTEXT_CALLBACK_BOOL
+PRECONDITION_CALLBACK = __SOURCE_CONTEXT_CALLBACK_BOOL
 
 
-class _ErrorHandler(NamedTuple):
+@dataclasses.dataclass(frozen=True)
+class _ErrorHandler:
 	callback: ERROR_HANDLER_CALLBACK
 	handled: bool
 
 
-class _Requirement(NamedTuple):
+@dataclasses.dataclass(frozen=True)
+class _Requirement:
 	requirement: REQUIRES_CALLBACK
 	failure_message_getter: Optional[FAIL_MSG_CALLBACK]
 
 
 _ERROR_HANDLER_TYPE = Dict[Type[CommandError], _ErrorHandler]
-Self = TypeVar('Self', bound='AbstractNode')
 
 
 class AbstractNode(ABC):
@@ -52,6 +78,7 @@ class AbstractNode(ABC):
 		self._callback: Optional[RUNS_CALLBACK] = None
 		self._error_handlers: _ERROR_HANDLER_TYPE = {}
 		self._child_error_handlers: _ERROR_HANDLER_TYPE = {}
+		self._preconditions: List[PRECONDITION_CALLBACK] = []
 		self._requirements: List[_Requirement] = []
 		self._redirect_node: Optional[AbstractNode] = None
 		self._suggestion_getter: SUGGESTS_CALLBACK = lambda: []
@@ -60,7 +87,7 @@ class AbstractNode(ABC):
 	#   Interfaces
 	# --------------
 
-	def then(self: Self, node: 'AbstractNode') -> Self:
+	def then(self, node: 'AbstractNode') -> Self:
 		"""
 		Attach a child node to its children list, and then return itself
 
@@ -83,7 +110,7 @@ class AbstractNode(ABC):
 		"""
 		if self._redirect_node is not None:
 			raise IllegalNodeOperation('Redirected node is not allowed to add child nodes')
-		class_util.check_type(node, AbstractNode)
+		class_utils.check_type(node, AbstractNode)
 		if isinstance(node, Literal):
 			for literal in node.literals:
 				self._children_literal[literal].append(node)
@@ -91,7 +118,7 @@ class AbstractNode(ABC):
 			self._children.append(node)
 		return self
 
-	def runs(self: Self, func: RUNS_CALLBACK) -> Self:
+	def runs(self, func: RUNS_CALLBACK) -> Self:
 		"""
 		Set the callback function of this node. When the command parsing finished at this node, the callback function will be executed
 
@@ -122,14 +149,14 @@ class AbstractNode(ABC):
 		:param func: A callable that accepts up to 2 arguments.
 			Argument list: :class:`~mcdreforged.command.command_source.CommandSource`, :class:`dict` (:class:`~mcdreforged.command.builder.common.CommandContext`)
 		"""
-		class_util.check_type(func, Callable)
+		class_utils.check_type(func, Callable)  # type: ignore  # see also: python/mypy#14928
 		self._callback = func
 		return self
 
-	def requires(self: Self, requirement: REQUIRES_CALLBACK, failure_message_getter: Optional[FAIL_MSG_CALLBACK] = None) -> Self:
+	def requires(self, requirement: REQUIRES_CALLBACK, failure_message_getter: Optional[FAIL_MSG_CALLBACK] = None) -> Self:
 		"""
 		Set the requirement tester callback of the node. When entering this node, MCDR will invoke the requirement tester
-		to see if the current command source and context match your specific condition.
+		to see if the current command source and context meet the specified condition
 
 		If the tester callback return True, nothing will happen, MCDR will continue parsing the rest of the command
 
@@ -148,19 +175,54 @@ class AbstractNode(ABC):
 
 		Example usages::
 
-			node1.requires(lambda src: src.has_permission(3))  # Permission check
+			node1.requires(lambda src: src.has_permission(3))  # Permission check, error if the permission is not enough
 			node2.requires(lambda src, ctx: ctx['page_count'] <= get_max_page())  # Dynamic range check
 			node3.requires(lambda src, ctx: is_legal(ctx['target']), lambda src, ctx: 'target {} is illegal'.format(ctx['target']))  # Customized failure message
 		"""
-		class_util.check_type(requirement, Callable)
-		class_util.check_type(failure_message_getter, [Callable, None])
+		class_utils.check_type(requirement, Callable)  # type: ignore  # see also: python/mypy#14928
+		class_utils.check_type(failure_message_getter, (Callable, None))  # type: ignore  # see also: python/mypy#14928
 		self._requirements.append(_Requirement(requirement, failure_message_getter))
 		return self
 
-	def redirects(self: Self, redirect_node: 'AbstractNode') -> Self:
+	def precondition(self, precondition: PRECONDITION_CALLBACK) -> Self:
+		"""
+		Set the precondition callback for this node. Before attempting to enter this node, MCDR will invoke the precondition callback
+		to see if the current command source and context meet the specified condition
+
+		If the precondition callback returns True, the node will remain accessible, and MCDR will continue entering the node
+
+		If the precondition callback returns False, the node will be filtered out and treated as if it does not exist
+
+		.. versionadded:: v2.14.0
+
+		:param precondition: A callable that accepts up to 2 arguments and returns a bool.
+			Argument list: :class:`~mcdreforged.command.command_source.CommandSource`, :class:`dict` (:class:`~mcdreforged.command.builder.common.CommandContext`)
+
+		Example usages::
+
+			node1.precondition(lambda src: src.has_permission(3))  # Permission check, but hide if the permission is not enough
+			node2.precondition(lambda src, ctx: 'foo' in ctx)  # Avoid re-assigning the "foo" argument
+		"""
+		class_utils.check_type(precondition, Callable)  # type: ignore  # see also: python/mypy#14928
+		self._preconditions.append(precondition)
+		return self
+
+	def redirects(self, redirect_node: 'AbstractNode') -> Self:
 		"""
 		Redirect all further child nodes command parsing to another given node
 
+		Redirected stuffs:
+
+		* Children node traversal
+		* Command callback, if current node doesn't have a callback
+
+		Unredirected stuffs:
+
+		* Command parsing, i.e. parsing the literal / argument value of the node from command
+		* Requirement testing
+		* Precondition testing
+		* Suggestion fetching
+
 		Example use cases:
 
 		* You want a short command and full-path command that will all execute the same commands
@@ -173,11 +235,11 @@ class AbstractNode(ABC):
 		"""
 		if self.has_children():
 			raise IllegalNodeOperation('Node with children nodes is not allowed to be redirected')
-		class_util.check_type(redirect_node, AbstractNode)
+		class_utils.check_type(redirect_node, AbstractNode)
 		self._redirect_node = redirect_node
 		return self
 
-	def suggests(self: Self, suggestion: SUGGESTS_CALLBACK) -> Self:
+	def suggests(self, suggestion: SUGGESTS_CALLBACK) -> Self:
 		"""
 		Set the provider for command suggestions of this node
 
@@ -197,11 +259,11 @@ class AbstractNode(ABC):
 		:param suggestion: A callable function which accepts up to 2 parameters and return an iterable of str indicating the current command suggestions.
 			Argument list: :class:`~mcdreforged.command.command_source.CommandSource`, :class:`dict` (:class:`~mcdreforged.command.builder.common.CommandContext`)
 		"""
-		class_util.check_type(suggestion, Callable)
+		class_utils.check_type(suggestion, Callable)  # type: ignore  # see also: python/mypy#14928
 		self._suggestion_getter = suggestion
 		return self
 
-	def on_error(self: Self, error_type: Type[CommandError], handler: ERROR_HANDLER_CALLBACK, *, handled: bool = False) -> Self:
+	def on_error(self, error_type: Type[CommandError], handler: ERROR_HANDLER_CALLBACK, *, handled: bool = False) -> Self:
 		"""
 		When a command error occurs, the given will invoke the given handler to handle with the error
 
@@ -214,19 +276,19 @@ class AbstractNode(ABC):
 		"""
 		if not issubclass(error_type, CommandError):
 			raise TypeError('error_type parameter should be a class inherited from CommandError, but class {} found'.format(error_type))
-		class_util.check_type(error_type, Type)
-		class_util.check_type(handler, Callable)
+		class_utils.check_type(error_type, type)
+		class_utils.check_type(handler, Callable)  # type: ignore  # see also: python/mypy#14928
 		self._error_handlers[error_type] = _ErrorHandler(handler, handled)
 		return self
 
-	def on_child_error(self: Self, error_type: Type[CommandError], handler: ERROR_HANDLER_CALLBACK, *, handled: bool = False) -> Self:
+	def on_child_error(self, error_type: Type[CommandError], handler: ERROR_HANDLER_CALLBACK, *, handled: bool = False) -> Self:
 		"""
 		Similar to :meth:`on_error`, but it gets triggered only when the node receives a command error from one of the node's direct or indirect child
 		"""
 		if not issubclass(error_type, CommandError):
 			raise TypeError('error_type parameter should be a class inherited from CommandError, but class {} found'.format(error_type))
-		class_util.check_type(error_type, Type)
-		class_util.check_type(handler, Callable)
+		class_utils.check_type(error_type, type)
+		class_utils.check_type(handler, Callable)  # type: ignore  # see also: python/mypy#14928
 		self._child_error_handlers[error_type] = _ErrorHandler(handler, handled)
 		return self
 
@@ -238,12 +300,15 @@ class AbstractNode(ABC):
 
 		.. versionadded:: v2.6.0
 		"""
-		tree_printer.print_tree(self, lambda node: node.get_children(), str, line_writer)
+		def children_getter(node: AbstractNode) -> List[AbstractNode]:
+			return node.get_children()
+		tree_printer.print_tree(self, children_getter, str, line_writer)
 
 	# -------------------
 	#   Interfaces ends
 	# -------------------
 
+	@abstractmethod
 	def _get_usage(self) -> str:
 		raise NotImplementedError()
 
@@ -251,12 +316,21 @@ class AbstractNode(ABC):
 		return len(self._children) + len(self._children_literal) > 0
 
 	def get_children(self) -> List['AbstractNode']:
-		children = []
+		children: List[AbstractNode] = []
 		for literal_list in self._children_literal.values():
 			children.extend(literal_list)
 		children.extend(self._children)
-		return misc_util.unique_list(children)
+		return collection_utils.unique_list(children)
 
+	def _on_visited(self, context: CommandContext, parsed_result: ParseResult):
+		"""
+		Invoked when this node is visited, right after the node successfully parses a command segment
+
+		A node can use the context dict to store its provided value here
+		"""
+		pass
+
+	@abstractmethod
 	def parse(self, text: str) -> ParseResult:
 		"""
 		Try to parse the text and get an argument
@@ -270,30 +344,18 @@ class AbstractNode(ABC):
 		raise NotImplementedError()
 
 	@staticmethod
-	def __smart_callback(callback: Callable, *args):
-		sig = inspect.signature(callback)
-		spec_args = inspect.getfullargspec(callback).args
-		spec_args_len = len(spec_args)
-		if isinstance(callback, MethodType):  # class method, remove the 1st param
-			spec_args_len -= 1
-		try:
-			sig.bind(*args[:spec_args_len])  # test if using full arg length is ok
-		except TypeError:
-			raise
-
+	def __smart_callback(callback: Callable, args: tuple, callback_error_factory: CallbackError.Builder):
 		# make sure all passed CommandContext are copies
-		args = list(args)
-		for i in range(len(args)):
-			arg = args[i]
-			if isinstance(arg, CommandContext):
-				args[i] = arg.copy()
-
-		return callback(*args[:spec_args_len])
+		adjusted_args = tuple(
+			arg.copy() if isinstance(arg, CommandContext) else arg
+			for arg in args
+		)
+		return ScheduledCallback(callback, adjusted_args, callback_error_factory).invoke(DirectCallbackInvoker())
 
 	def __handle_error(self, error: CommandError, context: CommandContext, error_handlers: _ERROR_HANDLER_TYPE):
 		for error_type, handler in error_handlers.items():
 			if isinstance(error, error_type):
-				self.__smart_callback(handler.callback, context.source, error, context)
+				self.__smart_callback(handler.callback, (context.source, error, context), CallbackError.builder(context, 'error handling'))
 				if handler.handled:
 					error.set_handled()
 
@@ -301,21 +363,29 @@ class AbstractNode(ABC):
 		self.__handle_error(error, context, self._error_handlers)
 		raise error
 
-	def __check_requirements(self, context: CommandContext) -> Optional[FAIL_MSG_CALLBACK]:
+	def __check_requirements(self, context: CommandContext) -> Optional[_Requirement]:
 		"""
-		:return: None: requirement check passed;
-		failure_message_getter: requirement check failed
+		:return: None: requirement check passed; otherwise, the unsatisfied requirement
 		"""
 		for req in self._requirements:
-			if not self.__smart_callback(req.requirement, context.source, context):
-				return req.failure_message_getter or (lambda: None)
+			ok = self.__smart_callback(req.requirement, (context.source, context), CallbackError.builder(context, 'requirements check'))
+			if not ok:
+				return req
 		return None
 
-	def _get_suggestions(self, context: CommandContext) -> Iterable[str]:
-		return self.__smart_callback(self._suggestion_getter, context.source, context)
+	def __check_preconditions(self, context: CommandContext) -> bool:
+		for precondition in self._preconditions:
+			ok = self.__smart_callback(precondition, (context.source, context), CallbackError.builder(context, 'preconditions check'))
+			if not ok:
+				return False
+		return True
 
-	def _execute_command(self, context: CommandContext) -> None:
-		command = context.command  # type: str
+	def _get_suggestions(self, context: CommandContext) -> Iterable[str]:
+		return self.__smart_callback(self._suggestion_getter, (context.source, context), CallbackError.builder(context, 'suggestions getting'))
+
+	def _execute_command(self, context: CommandContext) -> CommandExecutions:
+		command = context.command
+		executions = CommandExecutions()
 		try:
 			parse_result = self.parse(context.command_remaining)
 		except CommandSyntaxError as error:
@@ -323,13 +393,16 @@ class AbstractNode(ABC):
 			error.set_failed_command(context.command_read + context.command_remaining[:error.char_read])
 			self.__raise_error(error, context)
 		else:
-			next_remaining = utils.remove_divider_prefix(context.command_remaining[parse_result.char_read:])  # type: str
-			total_read = len(command) - len(next_remaining)  # type: int
+			next_remaining = utils.remove_divider_prefix(context.command_remaining[parse_result.char_read:])
+			total_read = len(command) - len(next_remaining)
 
-			with context.read_command(self, parse_result, total_read):
-				getter = self.__check_requirements(context)
-				if getter is not None:  # requirement check failed
-					failure_message = self.__smart_callback(getter, context.source, context)
+			with context.visit_node(self, parse_result, total_read):
+				req = self.__check_requirements(context)
+				if req is not None:  # requirement check failed
+					if req.failure_message_getter is not None:
+						failure_message = self.__smart_callback(req.failure_message_getter, (context.source, context), CallbackError.builder(context, 'failure message getting'))
+					else:
+						failure_message = None
 					self.__raise_error(RequirementNotMet(context.command_read, context.command_read, failure_message), context)
 
 				# Parsing finished
@@ -338,7 +411,11 @@ class AbstractNode(ABC):
 					if callback is None and self._redirect_node is not None:
 						callback = self._redirect_node._callback
 					if callback is not None:
-						self.__smart_callback(callback, context.source, context)
+						context2 = context.copy()
+						executions.append(CommandExecution(
+							context=context2,
+							scheduled_callback=ScheduledCallback(callback, (context2.source, context2), CallbackError.builder(context2, 'command callback')),
+						))
 					else:
 						self.__raise_error(UnknownCommand(context.command_read, context.command_read), context)
 				# Un-parsed command string remains
@@ -356,10 +433,13 @@ class AbstractNode(ABC):
 						try:
 							# Check literal children first
 							literal_error = None
+							child_literal: AbstractNode  # satisfy pycharm's static checker on the __check_preconditions() call
 							for child_literal in node._children_literal.get(next_literal, []):
+								if not child_literal.__check_preconditions(context):
+									continue
 								try:
 									with context.enter_child(child_literal):
-										child_literal._execute_command(context)
+										executions.extend(child_literal._execute_command(context))
 									break
 								except CommandError as e:
 									# it's ok for a direct literal node to fail
@@ -369,8 +449,13 @@ class AbstractNode(ABC):
 								if literal_error is not None:
 									raise literal_error
 								for child in node._children:
-									with context.enter_child(child):
-										child._execute_command(context)
+									if not child.__check_preconditions(context):
+										continue
+									try:
+										with context.enter_child(child):
+											executions.extend(child._execute_command(context))
+									except UnknownArgument:
+										continue
 									break
 								else:  # No argument child
 									argument_unknown = True
@@ -382,6 +467,8 @@ class AbstractNode(ABC):
 					if argument_unknown:
 						self.__raise_error(UnknownArgument(context.command_read, command), context)
 
+		return executions
+
 	def _generate_suggestions(self, context: CommandContext) -> CommandSuggestions:
 		"""
 		Return a list of tuple (suggested command, suggested argument)
@@ -400,11 +487,11 @@ class AbstractNode(ABC):
 		except CommandSyntaxError:
 			return self_suggestions()
 		else:
-			success_read = len(context.command) - len(context.command_remaining) + result.char_read  # type: int
-			next_remaining = utils.remove_divider_prefix(context.command_remaining[result.char_read:])  # type: str
-			total_read = len(context.command) - len(next_remaining)  # type: int
+			success_read = len(context.command) - len(context.command_remaining) + result.char_read
+			next_remaining = utils.remove_divider_prefix(context.command_remaining[result.char_read:])
+			total_read = len(context.command) - len(next_remaining)
 
-			with context.read_command(self, result, total_read):
+			with context.visit_node(self, result, total_read):
 				if self.__check_requirements(context) is not None:
 					return CommandSuggestions()
 
@@ -418,16 +505,23 @@ class AbstractNode(ABC):
 				node = self if self._redirect_node is None else self._redirect_node
 				# Check literal children first
 				children_literal = node._children_literal.get(utils.get_element(next_remaining), [])
+				child_literal: AbstractNode  # satisfy pycharm's static checker on the __check_preconditions() call
 				for child_literal in children_literal:
+					if not child_literal.__check_preconditions(context):
+						continue
 					with context.enter_child(child_literal):
 						suggestions.extend(child_literal._generate_suggestions(context))
 				if len(children_literal) == 0:
 					for literal_list in node._children_literal.values():
 						for child_literal in literal_list:
+							if not child_literal.__check_preconditions(context):
+								continue
 							with context.enter_child(child_literal):
 								suggestions.extend(child_literal._generate_suggestions(context))
 					usages = []
 					for child in node._children:
+						if not child.__check_preconditions(context):
+							continue
 						with context.enter_child(child):
 							suggestions.extend(child._generate_suggestions(context))
 							if len(next_remaining) == 0:
@@ -438,7 +532,7 @@ class AbstractNode(ABC):
 
 
 class EntryNode(AbstractNode, ABC):
-	def execute(self, source: CommandSource, command: str):
+	def _entry_execute(self, source: CommandSource, command: str) -> CommandExecutions:
 		"""
 		Parse and execute this command
 
@@ -450,12 +544,12 @@ class EntryNode(AbstractNode, ABC):
 		try:
 			context = CommandContext(source, command)
 			with context.enter_child(self):
-				self._execute_command(context)
+				return self._execute_command(context)
 		except LiteralNotMatch as error:
 			# the root literal node fails to parse the first element
 			raise UnknownRootArgument(error.get_parsed_command(), error.get_failed_command()) from error
 
-	def generate_suggestions(self, source: CommandSource, command: str) -> CommandSuggestions:
+	def _entry_generate_suggestions(self, source: CommandSource, command: str) -> CommandSuggestions:
 		"""
 		Get a list of command suggestion of given command
 
@@ -478,7 +572,7 @@ class Literal(EntryNode):
 
 	Literal node is the only node that can start a command execution
 	"""
-	def __init__(self, literal: str or Iterable[str]):
+	def __init__(self, literal: Union[str, Iterable[str]]):
 		super().__init__()
 		if isinstance(literal, str):
 			literals = {literal}
@@ -490,16 +584,19 @@ class Literal(EntryNode):
 			if not isinstance(literal, str):
 				raise TypeError('Literal node only accepts str but {} found'.format(type(literal)))
 			if utils.DIVIDER in literal:
-				raise TypeError('DIVIDER character "{}" cannot be inside a literal'.format(utils.DIVIDER))
-		self.literals = literals  # type: Set[str]
+				raise TypeError('DIVIDER character {!r} cannot be inside a literal'.format(utils.DIVIDER))
+		self.literals: Set[str] = literals
 		self._suggestion_getter = lambda: self.literals
 
+	@override
 	def _get_usage(self) -> str:
 		return '|'.join(sorted(self.literals))
 
-	def suggests(self, suggestion: SUGGESTS_CALLBACK) -> 'AbstractNode':
+	@override
+	def suggests(self, suggestion: SUGGESTS_CALLBACK) -> NoReturn:
 		raise IllegalNodeOperation('Literal node does not support suggests')
 
+	@override
 	def parse(self, text):
 		arg = utils.get_element(text)
 		if arg in self.literals:
@@ -511,7 +608,7 @@ class Literal(EntryNode):
 		return 'Literal {}'.format(repr(tuple(self.literals)[0]) if len(self.literals) == 1 else set(self.literals))
 
 	def __repr__(self):
-		return 'Literal[literals={}]'.format(self.literals)
+		return class_utils.represent(self, {'literals': self.literals})
 
 
 class ArgumentNode(AbstractNode, ABC):
@@ -520,18 +617,57 @@ class ArgumentNode(AbstractNode, ABC):
 
 	It has a str field ``name`` which is used as the key used in storing parsed value in context
 	"""
-	def __init__(self, name: str):
+
+	class _InitKwargs(TypedDict):
+		accumulate: NotRequired[bool]
+		metavar: NotRequired[str]
+
+	def __init__(
+			self, name: str, *,
+			accumulate: Optional[bool] = None,
+			metavar: Optional[str] = None,
+	):
+		"""
+		:param name: The name of the argument node. Should be unique within the command tree path
+		:keyword accumulate: If set to True, then the parsed value will be stored in a list in the command context.
+			Re-visiting of this node appends new value to the end of the list
+		:keyword metavar: Optional str. It behaves like the *metavar* kwarg of :external:meth:`argparse.ArgumentParser.add_argument`,
+			which overrides the name displayed of the command suggestion usage placeholder.
+			If not provided, the *name* argument will be used as the default placeholder
+
+		.. versionadded:: v2.13.0
+			The *accumulate* parameter
+		.. versionadded:: v2.14.0
+			The *metavar* parameter
+		"""
 		super().__init__()
 		self.__name = name
+		self.__accumulate: bool = bool(accumulate)
+		self.__metavar = metavar
+
+	@override
+	def _on_visited(self, context: CommandContext, parsed_result: ParseResult):
+		if self.__accumulate:
+			if self.__name not in context:
+				context[self.__name] = []
+			context[self.__name].append(parsed_result.value)
+		else:
+			context[self.__name] = parsed_result.value
 
 	def get_name(self) -> str:
 		return self.__name
 
+	@override
 	def _get_usage(self) -> str:
-		return '<{}>'.format(self.__name)
+		return '<{}>'.format(self.__metavar or self.__name)
 
 	def __str__(self):
 		return '{} <{}>'.format(self.__class__.__name__, self.get_name())
 
 	def __repr__(self):
-		return '{}[name={}]'.format(self.__class__.__name__, self.__name)
+		fields: dict = {'name': self.__name}
+		if self.__accumulate:
+			fields['accumulate'] = True
+		if self.__metavar:
+			fields['metavar'] = True
+		return class_utils.represent(self, fields)
