@@ -1,13 +1,15 @@
 from abc import ABC
-from typing import Dict, Callable, TYPE_CHECKING, Optional, List, TypeVar, Generic, Any, Type
+from typing import Dict, Callable, TYPE_CHECKING, Optional, List, TypeVar, Generic, Any, Type, overload, Union, cast
+
+from typing_extensions import Self, override
 
 from mcdreforged.command.builder.exception import CommandError
-from mcdreforged.command.builder.nodes.basic import RUNS_CALLBACK, Literal, AbstractNode, ArgumentNode, REQUIRES_CALLBACK, FAIL_MSG_CALLBACK, SUGGESTS_CALLBACK, ERROR_HANDLER_CALLBACK
+from mcdreforged.command.builder.nodes.basic import RUNS_CALLBACK, Literal, AbstractNode, ArgumentNode, REQUIRES_CALLBACK, FAIL_MSG_CALLBACK, SUGGESTS_CALLBACK, ERROR_HANDLER_CALLBACK, PRECONDITION_CALLBACK
 from mcdreforged.command.command_source import CommandSource
-from mcdreforged.utils import tree_printer
 
 if TYPE_CHECKING:
-	from mcdreforged.plugin.server_interface import PluginServerInterface
+	from mcdreforged.plugin.si.plugin_server_interface import PluginServerInterface
+	from mcdreforged.utils import tree_printer
 
 
 class Requirements:
@@ -69,7 +71,8 @@ class Requirements:
 
 NodeType = TypeVar('NodeType', bound=AbstractNode)
 ArgNodeType = TypeVar('ArgNodeType', bound=ArgumentNode)
-Self = TypeVar('Self', bound='NodeDefinition')
+CommandCallbackFunc = TypeVar('CommandCallbackFunc', bound=RUNS_CALLBACK)
+CommandCallbackDecorator = Callable[[CommandCallbackFunc], CommandCallbackFunc]
 
 
 class NodeDefinition(Generic[NodeType], ABC):
@@ -77,41 +80,49 @@ class NodeDefinition(Generic[NodeType], ABC):
 	A node definition class holding extra customization information
 	"""
 
-	def post_process(self: Self, post_processor: Callable[[NodeType], Any]) -> Self:
+	def post_process(self, post_processor: Callable[[NodeType], Any]) -> Self:
 		"""
-		Added a post-process function to the current node definition
+		Add a post-process function to the current node definition
 
 		During method :meth:`SimpleCommandBuilder.build`, after a node is constructed,
 		all the post-process functions will be applied to the node object
 		"""
 		raise NotImplementedError()
 
-	def requires(self: Self, requirement: REQUIRES_CALLBACK, failure_message_getter: Optional[FAIL_MSG_CALLBACK] = None) -> Self:
+	def requires(self, requirement: REQUIRES_CALLBACK, failure_message_getter: Optional[FAIL_MSG_CALLBACK] = None) -> Self:
 		"""
 		See :meth:`AbstractNode.requires() <mcdreforged.command.builder.nodes.basic.AbstractNode.requires>`
 		"""
 		raise NotImplementedError()
 
-	def suggests(self: Self, suggestion: SUGGESTS_CALLBACK) -> Self:
+	def precondition(self, precondition: PRECONDITION_CALLBACK) -> Self:
+		"""
+		See :meth:`AbstractNode.precondition() <mcdreforged.command.builder.nodes.basic.AbstractNode.precondition>`
+
+		.. versionadded:: v2.14.0
+		"""
+		raise NotImplementedError()
+
+	def suggests(self, suggestion: SUGGESTS_CALLBACK) -> Self:
 		"""
 		See :meth:`AbstractNode.suggests() <mcdreforged.command.builder.nodes.basic.AbstractNode.suggests>`
 		"""
 		raise NotImplementedError()
 
-	def on_error(self: Self, error_type: Type[CommandError], handler: ERROR_HANDLER_CALLBACK, *, handled: bool = False) -> Self:
+	def on_error(self, error_type: Type[CommandError], handler: ERROR_HANDLER_CALLBACK, *, handled: bool = False) -> Self:
 		"""
 		See :meth:`AbstractNode.on_error() <mcdreforged.command.builder.nodes.basic.AbstractNode.on_error>`
 		"""
 		raise NotImplementedError()
 
-	def on_child_error(self: Self, error_type: Type[CommandError], handler: ERROR_HANDLER_CALLBACK, *, handled: bool = False) -> Self:
+	def on_child_error(self, error_type: Type[CommandError], handler: ERROR_HANDLER_CALLBACK, *, handled: bool = False) -> Self:
 		"""
 		See :meth:`AbstractNode.on_child_error() <mcdreforged.command.builder.nodes.basic.AbstractNode.on_child_error>`
 		"""
 		raise NotImplementedError()
 
 
-class _NodeDefinitionImpl(NodeDefinition):
+class _NodeDefinitionImpl(NodeDefinition[NodeType]):
 	def __init__(self, node_factory: Callable[[str], NodeType]):
 		self.__node_factory = node_factory
 		self.__node_processors: List[Callable[[NodeType], Any]] = []
@@ -122,21 +133,40 @@ class _NodeDefinitionImpl(NodeDefinition):
 			processor(node)
 		return node
 
+	@override
 	def post_process(self, post_processor: Callable[[NodeType], Any]) -> Self:
 		self.__node_processors.append(post_processor)
 		return self
 
-	def requires(self: Self, requirement: REQUIRES_CALLBACK, failure_message_getter: Optional[FAIL_MSG_CALLBACK] = None) -> Self:
-		return self.post_process(lambda n: n.requires(requirement, failure_message_getter))
+	@override
+	def requires(self, requirement: REQUIRES_CALLBACK, failure_message_getter: Optional[FAIL_MSG_CALLBACK] = None) -> Self:
+		def post_processor_requires(node: NodeType):
+			node.requires(requirement, failure_message_getter)
+		return self.post_process(post_processor_requires)
 
-	def suggests(self: Self, suggestion: SUGGESTS_CALLBACK) -> Self:
-		return self.post_process(lambda n: n.suggests(suggestion))
+	@override
+	def precondition(self, precondition: PRECONDITION_CALLBACK) -> Self:
+		def post_processor_requires(node: NodeType):
+			node.requires(precondition)
+		return self.post_process(post_processor_requires)
 
-	def on_error(self: Self, error_type: Type[CommandError], handler: ERROR_HANDLER_CALLBACK, *, handled: bool = False) -> Self:
-		return self.post_process(lambda n: n.on_error(error_type, handler, handled=handled))
+	@override
+	def suggests(self, suggestion: SUGGESTS_CALLBACK) -> Self:
+		def post_processor_suggests(node: NodeType):
+			node.suggests(suggestion)
+		return self.post_process(post_processor_suggests)
 
-	def on_child_error(self: Self, error_type: Type[CommandError], handler: ERROR_HANDLER_CALLBACK, *, handled: bool = False) -> Self:
-		return self.post_process(lambda n: n.on_child_error(error_type, handler, handled=handled))
+	@override
+	def on_error(self, error_type: Type[CommandError], handler: ERROR_HANDLER_CALLBACK, *, handled: bool = False) -> Self:
+		def post_processor_on_error(node: NodeType):
+			node.on_error(error_type, handler, handled=handled)
+		return self.post_process(post_processor_on_error)
+
+	@override
+	def on_child_error(self, error_type: Type[CommandError], handler: ERROR_HANDLER_CALLBACK, *, handled: bool = False) -> Self:
+		def post_processor_on_child_error(node: NodeType):
+			node.on_child_error(error_type, handler, handled=handled)
+		return self.post_process(post_processor_on_child_error)
 
 
 class SimpleCommandBuilder:
@@ -171,9 +201,6 @@ class SimpleCommandBuilder:
 	def __strip_arg(cls, node_name: str) -> str:
 		return node_name[1:-1] if cls.__is_arg(node_name) else node_name
 
-	def __clean_cache(self):
-		self.__build_cache = None
-
 	def __locate_or_create_child(self, parent: AbstractNode, node_name: str) -> AbstractNode:
 		child_map: Dict[str, AbstractNode] = {}
 		for child in parent.get_children():
@@ -188,11 +215,12 @@ class SimpleCommandBuilder:
 
 		child = child_map.get(node_name)
 		if child is None:
-			child_factory: _NodeDefinitionImpl[AbstractNode]
+			child_factory: _NodeDefinitionImpl
 			if self.__is_arg(node_name):
-				child_factory = self.__arguments.get(node_name)
-				if child_factory is None:
-					raise self.Error('Undefined arg {}'.format(node_name))
+				try:
+					child_factory = self.__arguments[node_name]
+				except KeyError:
+					raise self.Error('Undefined arg {}'.format(node_name)) from None
 			else:
 				child_factory = self.__literals.get(node_name, self.__DEFAULT_LITERAL_DEFINITION)
 
@@ -205,7 +233,12 @@ class SimpleCommandBuilder:
 	#   Interfaces
 	# --------------
 
-	def command(self, command: str, callback: RUNS_CALLBACK):
+	@overload
+	def command(self, command: str) -> CommandCallbackDecorator: ...
+	@overload
+	def command(self, command: str, callback: RUNS_CALLBACK) -> None: ...
+
+	def command(self, command: str, callback: Optional[RUNS_CALLBACK] = None) -> Union[CommandCallbackDecorator, None]:
 		"""
 		Define a command and its callback
 
@@ -219,11 +252,40 @@ class SimpleCommandBuilder:
 		You need to give definitions of argument nodes with the :meth:`arg` method.
 		You can also define your custom literal nodes with the :meth:`literal` method
 
+		Examples::
+		
+			builder.command('!!email list', list_email)
+			builder.command('!!email remove <email_id>', remove_email)
+			builder.command('!!email send <player> <message>', send_email)
+			
+		Alternative, you can use this method as a decorator to decorate the callback function by passing the first argument::
+
+			# This syntactic sugar does the same thing as `builder.command('!!email list', list_email)`
+			@builder.command('!!email list')
+			def list_email(source: CommandSource, context: CommandContext):
+				pass
+
+			# You can even chain the decorator. All given commands will use function `foo_bar()` as the callback
+			@builder.command('!!email foo')
+			@builder.command('!!email bar')
+			def foo_bar(source: CommandSource, context: CommandContext):
+				pass
+
 		:param command: A command path string, e.g. ``"!!calc add <value_a> <value_b>"``
 		:param callback: The callback function of this command, which will be passed to :meth:`AbstractNode.then<mcdreforged.command.builder.nodes.basic.AbstractNode.then>`
+
+		.. versionadded:: v2.10.0
+			It can now be used as a decorator for callback functions
 		"""
+		if callback is None:  # decorator mode
+			def decorator(func):
+				self.command(command, func)
+				return func
+			return decorator
+
 		self.__commands[command] = callback
-		self.__clean_cache()
+		self.clean_cache()
+		return None
 
 	def arg(self, arg_name: str, node_factory: Callable[[str], ArgNodeType]) -> NodeDefinition[ArgNodeType]:
 		"""
@@ -246,8 +308,8 @@ class SimpleCommandBuilder:
 		if not self.__is_arg(arg_name):
 			arg_name = self.__make_arg(arg_name)
 		definition = _NodeDefinitionImpl(node_factory)
-		self.__arguments[arg_name] = definition
-		self.__clean_cache()
+		self.__arguments[arg_name] = cast(_NodeDefinitionImpl[ArgumentNode], definition)
+		self.clean_cache()
 		return definition
 
 	def literal(self, literal_name: str, node_factory: Optional[Callable[[str], Literal]] = None) -> NodeDefinition[Literal]:
@@ -265,27 +327,32 @@ class SimpleCommandBuilder:
 			node_factory = Literal
 		definition = _NodeDefinitionImpl(node_factory)
 		self.__literals[literal_name] = definition
-		self.__clean_cache()
+		self.clean_cache()
 		return definition
 
 	# --------------
 	#    Outputs
 	# --------------
 
-	def build(self) -> List[AbstractNode]:
+	def build(self, *, use_cache: bool = True) -> List[AbstractNode]:
 		"""
 		Build the command trees
 
 		Nodes with same name will be reused. e.g. if you define 3 commands with path ``"!!foo"``, ``"!!foo bar"`` and "``!!foo baz"``,
 		the root ``"!!foo"`` node will be reused, and there will be only 1 ``"!!foo"`` node eventually
 
-		:return: A list of the built command tree root nodes. The result is cached until you instruct the builder again
+		:keyword use_cache: If set to false, do not use cache and always build new command trees
+		:return: A list of the built command tree root nodes. The result is cached until you instruct the builder again.
+			You can use :meth:`clean_cache` to explicitly clean the build cache
 		:raise SimpleCommandBuilder.Error: if there are undefined argument nodes
+
+		.. versionadded:: v2.9.0
+			Added ``use_cache`` keyword argument
 		"""
-		if self.__build_cache is None:
+		if not use_cache or self.__build_cache is None:
 			root = Literal('#TEMP')
 			for command, callback in self.__commands.items():
-				node = root
+				node: AbstractNode = root
 				for segment in command.split(' '):
 					if len(segment) > 0:
 						node = self.__locate_or_create_child(node, segment)
@@ -293,20 +360,41 @@ class SimpleCommandBuilder:
 			self.__build_cache = root.get_children()
 		return self.__build_cache
 
+	def clean_cache(self):
+		"""
+		Clean the build cache
+
+		See method :meth:`build`
+		"""
+		self.__build_cache = None
+
 	def register(self, server: 'PluginServerInterface'):
 		"""
 		A helper method for lazyman, to build with method :meth:`build` and register built commands to the MCDR server
 
-		:param server: The :class:`~mcdreforged.plugin.server_interface.PluginServerInterface` object of your plugin
-		:raise SimpleCommandBuilder.Error: if build fails, or there are rooted non-literal nodes
+		:param server: The :class:`~mcdreforged.plugin.si.plugin_server_interface.PluginServerInterface` object of your plugin
+		:raise SimpleCommandBuilder.Error: if build fails, or there are rooted non-literal nodes in the build result
 		"""
 		for node in self.build():
 			if isinstance(node, Literal):
 				server.register_command(node)
 			else:
-				raise self.Error('Not-literal root node is not supported'.format(node))
+				raise self.Error('Not-literal root node {} is not supported'.format(type(node)))
 
-	def print_tree(self, line_writer: tree_printer.LineWriter):
+	def add_children_for(self, parent_node: AbstractNode):
+		"""
+		A helper method for lazyman, to build command trees with method :meth:`build`
+		and attach all built tree nodes as the children of the given node via :meth:`~mcdreforged.command.builder.nodes.basic.AbstractNode.then`
+
+		:param parent_node: The command node that will become the parent of the built children nodes
+		:raise SimpleCommandBuilder.Error: if build fails
+
+		.. versionadded:: v2.8.0
+		"""
+		for node in self.build():
+			parent_node.then(node)
+
+	def print_tree(self, line_writer: 'tree_printer.LineWriter'):
 		"""
 		A helper method for lazyman, to build with method :meth:`build` and print the built command trees
 
