@@ -1,16 +1,19 @@
-from abc import ABC
+from abc import ABC, abstractmethod
 from contextlib import contextmanager
 from typing import TYPE_CHECKING, Optional
 
+from typing_extensions import override
+
 from mcdreforged.permission.permission_level import PermissionLevel
 from mcdreforged.translation.translation_text import RTextMCDRTranslation
-from mcdreforged.utils import misc_util
-from mcdreforged.utils.types import MessageText
+from mcdreforged.utils import misc_utils, class_utils
+from mcdreforged.utils.exception import IllegalCallError
+from mcdreforged.utils.types.message import MessageText
 
 if TYPE_CHECKING:
 	from mcdreforged.mcdr_server import MCDReforgedServer
 	from mcdreforged.info_reactor.info import Info
-	from mcdreforged.plugin.server_interface import ServerInterface
+	from mcdreforged.plugin.si.server_interface import ServerInterface
 	from mcdreforged.plugin.type.plugin import AbstractPlugin
 	from mcdreforged.preference.preference_manager import PreferenceItem
 
@@ -37,7 +40,7 @@ class CommandSource(ABC):
 
 		:return: ``True`` if it's a player command source, ``False`` otherwise
 		"""
-		raise NotImplementedError()
+		return isinstance(self, PlayerCommandSource)
 
 	@property
 	def is_console(self) -> bool:
@@ -46,14 +49,16 @@ class CommandSource(ABC):
 
 		:return: ``True`` if it's a console command source, ``False`` otherwise
 		"""
-		raise NotImplementedError()
+		return isinstance(self, ConsoleCommandSource)
 
+	@abstractmethod
 	def get_server(self) -> 'ServerInterface':
 		"""
 		Return the server interface instance
 		"""
 		raise NotImplementedError()
 
+	@abstractmethod
 	def get_permission_level(self) -> int:
 		"""
 		Return the permission level that the command source has
@@ -62,19 +67,28 @@ class CommandSource(ABC):
 		"""
 		raise NotImplementedError()
 
-	def get_preference(self) -> Optional['PreferenceItem']:
+	def get_preference(self) -> 'PreferenceItem':
 		"""
 		Return the preference of the command source
 
-		Only :class:`PlayerCommandSource` and :class:`ConsoleCommandSource` are supported, otherwise None will be returned
+		By default, the default preference of MCDR from
+		:meth:`ServerInterface.get_default_preference() <mcdreforged.plugin.si.server_interface.ServerInterface.get_preference>`
+		will be returned
+
+		Subclasses might override this method to return customized preference.
+		e.g. :class:`PlayerCommandSource` will return the preference of the corresponding player
 
 		.. seealso::
 
-			:class:`~mcdreforged.plugin.server_interface.ServerInterface`'s method :meth:`~mcdreforged.plugin.server_interface.ServerInterface.get_preference`
+			method :meth:`ServerInterface.get_preference() <mcdreforged.plugin.si.server_interface.ServerInterface.get_preference>`
 
 		.. versionadded:: v2.1.0
 		"""
-		return None
+		from mcdreforged.plugin.si.server_interface import ServerInterface
+		server = ServerInterface.get_instance()
+		if server is None:
+			raise IllegalCallError('Cannot get default preference when MCDR is not running')
+		return server.get_default_preference()
 
 	@contextmanager
 	def preferred_language_context(self):
@@ -115,6 +129,7 @@ class CommandSource(ABC):
 		"""
 		return self.get_permission_level() > level
 
+	@abstractmethod
 	def reply(self, message: MessageText, **kwargs) -> None:
 		"""
 		Send a message to the command source. The message can be anything including RTexts
@@ -125,6 +140,14 @@ class CommandSource(ABC):
 		"""
 		raise NotImplementedError()
 
+	def __eq__(self, other) -> bool:
+		"""
+		.. versionadded:: v2.12.2
+
+			All MCDR builtin command sources have the implemented the `__eq__` method
+		"""
+		return super().__eq__(other)
+
 
 class InfoCommandSource(CommandSource, ABC):
 	"""
@@ -140,9 +163,11 @@ class InfoCommandSource(CommandSource, ABC):
 		"""
 		return self.__info
 
+	@override
 	def get_server(self) -> 'ServerInterface':
 		return self._mcdr_server.basic_server_interface
 
+	@override
 	def get_permission_level(self) -> int:
 		return self._mcdr_server.permission_manager.get_permission(self)
 
@@ -154,7 +179,10 @@ class InfoCommandSource(CommandSource, ABC):
 
 
 class PlayerCommandSource(InfoCommandSource):
-	def __init__(self, mcdr_server, info, player: str):
+	"""
+	Player command source: the command source that represents a player
+	"""
+	def __init__(self, mcdr_server: 'MCDReforgedServer', info: 'Info', player: str):
 		if not info.is_from_server:
 			raise TypeError('{} should be built from server info'.format(self.__class__.__name__))
 		super().__init__(mcdr_server, info)
@@ -162,47 +190,44 @@ class PlayerCommandSource(InfoCommandSource):
 		self.player: str = player
 		"""The name of the player"""
 
-	@property
-	def is_player(self) -> bool:
-		return True
-
-	@property
-	def is_console(self) -> bool:
-		return False
-
-	def get_preference(self) -> Optional['PreferenceItem']:
+	@override
+	def get_preference(self) -> 'PreferenceItem':
 		return self.get_server().get_preference(self)
 
+	@override
 	def reply(self, message: MessageText, *, encoding: Optional[str] = None, **kwargs):
 		"""
 		:keyword encoding: encoding method to be used in :meth:`ServerInterface.tell`
 		"""
 		self._mcdr_server.basic_server_interface.tell(self.player, message, encoding=encoding)
 
+	def __eq__(self, other) -> bool:
+		return isinstance(other, PlayerCommandSource) and self.player == other.player
+
 	def __str__(self):
 		return 'Player {}'.format(self.player)
 
 	def __repr__(self):
-		return '{}[player={},info_id={}]'.format(type(self).__name__, self.player, self.get_info().id)
+		return class_utils.represent(self, {
+			'player': self.player,
+			'info_id': self.get_info().id,
+		})
 
 
 class ConsoleCommandSource(InfoCommandSource):
-	def __init__(self, mcdr_server, info):
+	"""
+	Console command source: the command source that represents the console
+	"""
+	def __init__(self, mcdr_server: 'MCDReforgedServer', info: 'Info'):
 		if not info.is_from_console:
 			raise TypeError('{} should be built from console info'.format(self.__class__.__name__))
 		super().__init__(mcdr_server, info)
 
-	@property
-	def is_player(self) -> bool:
-		return False
-
-	@property
-	def is_console(self) -> bool:
-		return True
-
-	def get_preference(self) -> Optional['PreferenceItem']:
+	@override
+	def get_preference(self) -> 'PreferenceItem':
 		return self.get_server().get_preference(self)
 
+	@override
 	def reply(self, message: MessageText, *, console_text: Optional[MessageText] = None, **kwargs):
 		"""
 		:keyword console_text: If it's specified, overwrite the value of parameter ``message`` with it
@@ -210,40 +235,54 @@ class ConsoleCommandSource(InfoCommandSource):
 		if console_text is not None:
 			message = console_text
 		with self.preferred_language_context():
-			misc_util.print_text_to_console(self._mcdr_server.logger, message)
+			misc_utils.print_text_to_console(self._mcdr_server.logger, message)
+
+	def __eq__(self, other) -> bool:
+		return isinstance(other, ConsoleCommandSource)
 
 	def __str__(self):
 		return 'Console'
 
 	def __repr__(self):
-		return '{}[info_id={}]'.format(type(self).__name__, self.get_info().id)
+		return class_utils.represent(self, {
+			'info_id': self.get_info().id,
+		})
 
 
 class PluginCommandSource(CommandSource):
+	"""
+	Plugin command source: the command source that represents a MCDR plugin
+	"""
 	def __init__(self, server: 'ServerInterface', plugin: Optional['AbstractPlugin'] = None):
 		self.__server = server.as_basic_server_interface()
 		self.__logger = self.__server.logger
 		self.__plugin = plugin
 
-	@property
-	def is_player(self) -> bool:
-		return False
-
-	@property
-	def is_console(self) -> bool:
-		return False
-
+	@override
 	def get_server(self) -> 'ServerInterface':
 		return self.__server
 
+	@override
 	def get_permission_level(self) -> int:
 		return PermissionLevel.PLUGIN_LEVEL
 
+	@override
 	def reply(self, message: MessageText, **kwargs) -> None:
-		misc_util.print_text_to_console(self.__logger, message)
+		misc_utils.print_text_to_console(self.__logger, message)
+
+	def __eq__(self, other) -> bool:
+		return (
+				isinstance(other, PluginCommandSource) and
+				(
+						(self.__plugin is None and other.__plugin is None) or
+						(self.__plugin is not None and other.__plugin is not None and self.__plugin.get_id() == other.__plugin.get_id())
+				)
+		)
 
 	def __str__(self):
 		return 'Plugin' if self.__plugin is None else 'Plugin {}'.format(self.__plugin)
 
 	def __repr__(self):
-		return '{}[plugin={}]'.format(type(self).__name__, self.__plugin)
+		return class_utils.represent(self, {
+			'plugin': self.__plugin,
+		})
