@@ -1,6 +1,10 @@
 """
 Constants related to plugin for MCDR
 """
+from mcdreforged.constants import core_constant
+
+MCDR_PLUGIN_ID = core_constant.PACKAGE_NAME
+MCDR_PLUGIN_VERSION = core_constant.VERSION
 
 # the directory inside MCDR working directory that stores plugins' configuration files
 PLUGIN_CONFIG_DIRECTORY = 'config'
@@ -14,6 +18,9 @@ PACKED_PLUGIN_FILE_SUFFIXES = ('.mcdr', '.pyz')
 # The name of the meta file inside a packed plugin
 PLUGIN_META_FILE = 'mcdreforged.plugin.json'
 
+# The name of the link file for a LinkedDirectoryPlugin
+LINK_DIRECTORY_PLUGIN_FILE_NAME = 'mcdreforged.linked_directory_plugin.json'
+
 # The name of the python package requirement file
 PLUGIN_REQUIREMENTS_FILE = 'requirements.txt'
 
