@@ -1,3 +1,7 @@
+import re
+
+from typing_extensions import override
+
 from mcdreforged.handler.impl.bukkit_handler import BukkitHandler
 from mcdreforged.handler.impl.vanilla_handler import VanillaHandler
 
@@ -11,5 +15,6 @@ class Bukkit14Handler(BukkitHandler):
 	# paper is not included
 
 	@classmethod
-	def get_content_parsing_formatter(cls):
+	@override
+	def get_content_parsing_formatter(cls) -> re.Pattern:
 		return VanillaHandler.get_content_parsing_formatter()
