@@ -1,6 +1,10 @@
+import re
+
+from typing_extensions import override
+
 from mcdreforged.handler.impl.bukkit_handler import BukkitHandler
 from mcdreforged.handler.impl.vanilla_handler import VanillaHandler
-from mcdreforged.utils import string_util
+from mcdreforged.utils import string_utils
 
 
 class CatServerHandler(BukkitHandler):
@@ -14,13 +18,15 @@ class CatServerHandler(BukkitHandler):
 
 	@staticmethod
 	def __cleaned_info(info):
-		info.content = string_util.clean_minecraft_color_code(info.content)
+		info.content = string_utils.clean_minecraft_color_code(info.content)
 		return info
 
 	@classmethod
-	def get_content_parsing_formatter(cls):
+	@override
+	def get_content_parsing_formatter(cls) -> re.Pattern:
 		return VanillaHandler.get_content_parsing_formatter()
 
+	@override
 	def parse_player_left(self, info):
 		# §eSteve left the game§r
 		return super().parse_player_left(self.__cleaned_info(info))
