@@ -1,8 +1,8 @@
+import re
 from typing import Optional
 
-from parse import parse
+from typing_extensions import override
 
-from mcdreforged.handler.impl.bukkit_handler import BukkitHandler
 from mcdreforged.handler.impl.bungeecord_handler import BungeecordHandler
 from mcdreforged.info_reactor.info import Info
 
@@ -17,21 +17,26 @@ class WaterfallHandler(BungeecordHandler):
 	# [02:18:30 INFO]: Enabled plugin cmd_list version git:cmd_list:1.15-SNAPSHOT:f1c32f8:1489 by SpigotMC
 	# [02:18:29 INFO] [ViaVersion]: Loading 1.12.2 -> 1.13 mappings..."
 	@classmethod
-	def get_content_parsing_formatter(cls):
-		return (
-			BukkitHandler.get_content_parsing_formatter(),
-			'[{hour:d}:{min:d}:{sec:d} {logging}] {dummy}: {content}'  # something there is an extra element after the heading [] and :
+	@override
+	def get_content_parsing_formatter(cls) -> re.Pattern:
+		return re.compile(
+			r'\[(?P<hour>\d+):(?P<min>\d+):(?P<sec>\d+) (?P<logging>[^]]+)]'
+			r'( \[[^]]+])?'  # useless logger name
+			r': (?P<content>.*)'
 		)
 
+	@override
 	def parse_player_joined(self, info: Info) -> Optional[str]:
 		# [02:18:52 INFO]: [/127.0.0.1:14426] <-> InitialHandler has connected
 		# sadly no player id display here
 		return None
 
+	# [/127.0.0.1:14426|Fallen_Breath] -> UpstreamBridge has disconnected
+	__player_left_regex = re.compile(r'\[/[^|]+\|(?P<name>[^]]+)] -> UpstreamBridge has disconnected')
+
+	@override
 	def parse_player_left(self, info):
-		# [/127.0.0.1:14426|Fallen_Breath] -> UpstreamBridge has disconnected
 		if not info.is_user:
-			parsed = parse('[/{ip}|{name}] -> UpstreamBridge has disconnected', info.content)
-			if parsed is not None:
-				return parsed['name']
+			if (m := self.__player_left_regex.fullmatch(info.content)) is not None:
+				return m['name']
 		return None
