@@ -1,22 +1,19 @@
 from enum import Enum, unique, Flag, auto
-from typing import Iterable
+
+from typing_extensions import Self
 
 
 @unique
 class MCDReforgedFlag(Flag):
-	NONE = auto()
+	NONE = auto()               # placeholder
 	INTERRUPT = auto()			# ctrl-c flag
 	SERVER_STARTUP = auto()		# set to True after server startup
 	SERVER_RCON_READY = auto() 	# set to True after server started its rcon. used to start the rcon server
-	EXIT_AFTER_STOP = auto()	# if MCDR exit after server stop. can be modified by plugins
+	EXIT_AFTER_STOP = auto() 	# if MCDR exit after server stop. can be modified by plugins
 
 
 class EnumStateBase(Enum):
-	def in_state(self, states):
-		if type(states) is type(self):
-			return self is states
-		elif not isinstance(states, Iterable):
-			states = (states,)
+	def in_state(self: Self, *states: Self) -> bool:
 		return self in states
 
 
@@ -34,7 +31,3 @@ class ServerState(EnumStateBase):
 	STOPPED		= 'server_state.stopped'		# Server is stopped
 	STOPPING	= 'server_state.stopping'		# Server is being stopped by MCDR
 	RUNNING		= 'server_state.running'		# Server is running
-	# PRE_STOPPED	= 'server_state.pre_stopped' 	# Server is stopped, and it's going to dispatch_event SERVER_STOP
-
-	def is_server_stopped(self):
-		return self.in_state({self.STOPPED})
