@@ -1,6 +1,8 @@
 """
 Simple rcon client implement
+Reference: https://wiki.vg/RCON
 """
+import dataclasses
 import socket
 import struct
 import time
@@ -9,22 +11,26 @@ from threading import RLock
 from typing import Optional
 
 
-class PacketType:
+class _RequestId:
+	DEFAULT = 0
+	LOGIN_FAIL = -1
+
+
+class _PacketType:
 	COMMAND_RESPONSE = 0
 	COMMAND_REQUEST = 2
 	LOGIN_REQUEST = 3
-	LOGIN_FAIL = -1
 	ENDING_PACKET = 100
 
 
+@dataclasses.dataclass(frozen=True)
 class Packet:
-	def __init__(self, packet_type=None, payload=None):
-		self.packet_id = 0
-		self.packet_type = packet_type
-		self.payload = payload
+	request_id: int
+	packet_type: int
+	payload: str
 
-	def flush(self):
-		data = struct.pack('<ii', self.packet_id, self.packet_type) + bytes(self.payload + '\x00\x00', encoding='utf8')
+	def flush(self) -> bytes:
+		data = struct.pack('<ii', self.request_id, self.packet_type) + bytes(self.payload + '\x00\x00', encoding='utf8')
 		return struct.pack('<i', len(data)) + data
 
 
@@ -49,31 +55,32 @@ class RconConnection:
 		self.address = address
 		self.port = port
 		self.password = password
-		self.socket = None
+		self.socket: Optional[socket.socket] = None
 		self.command_lock = RLock()
 
 	def __del__(self):
 		self.disconnect()
 
-	def __send(self, data):
-		if type(data) is Packet:
-			data = data.flush()
-		self.socket.send(data)
+	def __send(self, data: Packet):
+		assert self.socket is not None
+		self.socket.send(data.flush())
 		time.sleep(0.03)  # MC-72390
 
-	def __receive(self, length):
+	def __receive(self, length: int) -> bytes:
+		assert self.socket is not None
 		data = bytes()
 		while len(data) < length:
 			data += self.socket.recv(min(self.BUFFER_SIZE, length - len(data)))
 		return data
 
-	def __receive_packet(self):
+	def __receive_packet(self) -> Packet:
 		length = struct.unpack('<i', self.__receive(4))[0]
 		data = self.__receive(length)
-		packet = Packet()
-		packet.packet_id = struct.unpack('<i', data[0:4])[0]
-		packet.packet_type = struct.unpack('<i', data[4:8])[0]
-		packet.payload = data[8:-2].decode('utf8')
+		packet = Packet(
+			request_id=struct.unpack('<i', data[0:4])[0],
+			packet_type=struct.unpack('<i', data[4:8])[0],
+			payload=data[8:-2].decode('utf8'),
+		)
 		return packet
 
 	def connect(self) -> bool:
@@ -85,12 +92,12 @@ class RconConnection:
 		if self.socket is not None:
 			try:
 				self.disconnect()
-			except:
+			except Exception:
 				pass
 		self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
 		self.socket.connect((self.address, self.port))
-		self.__send(Packet(PacketType.LOGIN_REQUEST, self.password))
-		success = self.__receive_packet().packet_id != PacketType.LOGIN_FAIL
+		self.__send(Packet(_RequestId.DEFAULT, _PacketType.LOGIN_REQUEST, self.password))
+		success = self.__receive_packet().request_id != _RequestId.LOGIN_FAIL
 		if not success:
 			self.disconnect()
 		return success
@@ -115,30 +122,32 @@ class RconConnection:
 		with self.command_lock:
 			for i in range(max_retry_time):
 				try:
-					self.__send(Packet(PacketType.COMMAND_REQUEST, command))
-					self.__send(Packet(PacketType.ENDING_PACKET, 'lol'))
+					self.__send(Packet(_RequestId.DEFAULT, _PacketType.COMMAND_REQUEST, command))
+					self.__send(Packet(_RequestId.DEFAULT, _PacketType.ENDING_PACKET, 'lol'))
 					result = ''
 					while True:
 						packet = self.__receive_packet()
-						if packet.payload == 'Unknown request {}'.format(hex(PacketType.ENDING_PACKET)[2:]):
+						if packet.payload == 'Unknown request {}'.format(hex(_PacketType.ENDING_PACKET)[2:]):
 							break
 						result += packet.payload
 					return result
-				except:
+				except Exception:
 					if self.logger is not None:
 						self.logger.warning('Rcon Fail to received packet')
 					try:
 						self.disconnect()
 						if self.connect():  # next try
 							continue
-					except:
+					except Exception:
 						pass
 					break
 		return None
 
 
 if __name__ == '__main__':
-	rcon = RconConnection('localhost', 25575, 'password')
-	print('Login success? ', rcon.connect())
-	while True:
-		print('Server ->', rcon.send_command(input('Server <- ')))
+	rcon = RconConnection('localhost', 25575, 'rcon_34ft786cbsqd')
+	ok = rcon.connect()
+	print('Login success: {}'.format(ok))
+	if ok:
+		while True:
+			print('Server ->', rcon.send_command(input('Server <- ')))
