@@ -1,9 +1,10 @@
+import dataclasses
 from contextlib import contextmanager
 from enum import unique, Enum, auto
-from typing import Dict, List, TYPE_CHECKING, NamedTuple, Optional, Set, TypeVar, Generic
+from typing import Dict, List, TYPE_CHECKING, Optional, Set, TypeVar, Generic
 
+from mcdreforged.logging.debug_option import DebugOption
 from mcdreforged.plugin.meta.version import VersionRequirement
-from mcdreforged.utils.logger import DebugOption
 
 if TYPE_CHECKING:
 	from mcdreforged.plugin.plugin_manager import PluginManager
@@ -67,23 +68,30 @@ class VisitingState(Enum):
 	FAIL = auto()
 
 
+@dataclasses.dataclass(frozen=True)
 class DependencyGraphNode:
-	def __init__(self, plugin_id: str):
-		self.plugin_id = plugin_id
-		self.parents: List['DependencyGraphNode'] = []
-		self.children: List['DependencyGraphNode'] = []
+	plugin_id: str
+	parents: List['DependencyGraphNode'] = dataclasses.field(default_factory=list)
+	children: List['DependencyGraphNode'] = dataclasses.field(default_factory=list)
 
 
+@dataclasses.dataclass
 class PluginHolder:
-	def __init__(self, plugin_id: str):
-		self.plugin_id = plugin_id
-		self.state = VisitingState.UNVISITED
-		self.graph_node: DependencyGraphNode = DependencyGraphNode(plugin_id)
-		self.topo_order: int = -1
-		self.error: Optional[DependencyError] = None
+	plugin_id: str
+	state: VisitingState = VisitingState.UNVISITED
+	graph_node: DependencyGraphNode = dataclasses.field(init=False)
+	topo_order: int = -1
+	error: Optional[DependencyError] = None
+
+	def __post_init__(self):
+		self.graph_node = DependencyGraphNode(self.plugin_id)
+
+	def __lt__(self, other: 'PluginHolder') -> bool:
+		return (self.topo_order, self.plugin_id) < (other.topo_order, other.plugin_id)
 
 
-class WalkResult(NamedTuple):
+@dataclasses.dataclass(frozen=True)
+class WalkResult:
 	plugin_id: str
 	success: bool
 	reason: Optional[DependencyError]
@@ -92,7 +100,7 @@ class WalkResult(NamedTuple):
 class DependencyWalker:
 	def __init__(self, plugin_manager: 'PluginManager'):
 		self.__plugin_manager = plugin_manager
-		self.__tr = plugin_manager.mcdr_server.tr
+		self.__tr = plugin_manager.mcdr_server.create_internal_translator('dependency_walker').tr
 		self.__holders: Dict[str, PluginHolder] = {}
 		self.__loop_checker: LoopChecker[str] = LoopChecker()
 		self.__topo_order: List[str] = []
@@ -115,23 +123,23 @@ class DependencyWalker:
 	def __ensure_loaded(self, plugin_id: str, requirement: Optional[VersionRequirement]):
 		holder = self.__get_holder(plugin_id)
 		if holder.state == VisitingState.FAIL:
-			raise DependencyParentFail(self.__tr('dependency_walker.dependency_parent_failed', plugin_id))
+			raise DependencyParentFail(self.__tr('dependency_parent_failed', plugin_id))
 
 		loop_list = self.__loop_checker.check(plugin_id)
 		if loop_list is not None:
 			loop_message = ' -> '.join(loop_list)
-			raise DependencyLoop(self.__tr('dependency_walker.dependency_loop', plugin_id, loop_message))
+			raise DependencyLoop(self.__tr('dependency_loop', plugin_id, loop_message))
 
 		with self.__loop_checker.enter(plugin_id):
 			plugin = self.__plugin_manager.get_plugin_from_id(plugin_id)
 			if plugin is None:
-				raise DependencyNotFound(self.__tr('dependency_walker.dependency_not_found', plugin_id))
+				raise DependencyNotFound(self.__tr('dependency_not_found', plugin_id))
 			plugin_name_display = plugin.get_name()
-			plugin_version = plugin.get_metadata().version
+			plugin_version = plugin.get_version()
 			plugin_dependencies = plugin.get_metadata().dependencies
 
 			if requirement is not None and isinstance(requirement, VersionRequirement) and not requirement.accept(plugin_version):
-				raise DependencyNotMet(self.__tr('dependency_walker.dependency_not_met', plugin_name_display, requirement))
+				raise DependencyNotMet(self.__tr('dependency_not_met', plugin_name_display, requirement))
 
 			if holder.state == VisitingState.PASS:  # no need to do further dependencies check, already done
 				return
@@ -143,9 +151,9 @@ class DependencyWalker:
 					self.__ensure_loaded(dep_id, req)
 				except DependencyError as e:
 					holder.state = VisitingState.FAIL
-					self.__plugin_manager.logger.debug('Set visiting state of {} to FAIL due to "{}"'.format(plugin_id, e), option=DebugOption.PLUGIN)
+					self.__plugin_manager.logger.mdebug('Set visiting state of {!r} to FAIL: {}'.format(plugin_id, e), option=DebugOption.PLUGIN)
 					raise
-			if not plugin.is_permanent():
+			if not plugin.is_builtin():
 				holder.topo_order = len(self.__topo_order)
 				self.__topo_order.append(plugin_id)
 			holder.state = VisitingState.PASS
@@ -167,7 +175,7 @@ class DependencyWalker:
 				if holder.state is not VisitingState.FAIL:
 					self.__ensure_loaded(plugin_id, None)
 				else:
-					raise DependencyError(self.__tr('dependency_walker.dependency_already_failed', plugin_id, holder.state))
+					raise DependencyError(self.__tr('dependency_already_failed', plugin_id, holder.state))
 			except DependencyError as e:
 				holder.error = e
 
@@ -212,13 +220,11 @@ class DependencyWalker:
 				get_children.extend(search(child_node))
 			return get_children
 
-		return list(filter(
-			lambda pid: pid in self.__walk_plugin_ids,
-			map(
-				lambda h: h.plugin_id,
-				sorted(
-					map(lambda n: self.__get_holder(n.plugin_id), search(holder.graph_node)),
-					key=lambda h: h.topo_order
-				)
-			)
-		))
+		return [
+			h.plugin_id
+			for h in sorted([
+				self.__get_holder(node.plugin_id)
+				for node in search(holder.graph_node)
+			])
+			if h.plugin_id in self.__walk_plugin_ids
+		]
