@@ -7,8 +7,8 @@ from typing import List, Dict, TYPE_CHECKING, Optional, Union
 from mcdreforged.minecraft.rtext.text import RTextBase, RText
 from mcdreforged.plugin.meta.version import Version, VersionParsingError, VersionRequirement
 from mcdreforged.translation.translation_text import RTextMCDRTranslation
-from mcdreforged.utils import translation_util, class_util
-from mcdreforged.utils.types import TranslationLanguageDict
+from mcdreforged.utils import translation_utils, class_utils
+from mcdreforged.utils.types.message import TranslationLanguageDict
 
 if TYPE_CHECKING:
 	from mcdreforged.plugin.type.plugin import AbstractPlugin
@@ -20,7 +20,13 @@ class Metadata:
 	"""
 
 	id: str
-	"""The id of the plugin. Should match regexp ``[a-z0-9_]{1,64}``"""
+	"""
+	The id of the plugin. Should match regexp ``[a-z][a-z0-9_]{0,63}``
+	
+	.. versionchanged:: v2.11.0
+		
+		Plugin id starts with non-alphabet character is no longer allowed
+	"""
 
 	version: Version
 	"""The version of the plugin, in a less restrictive semver format"""
@@ -46,7 +52,7 @@ class Metadata:
 	A dict of dependencies the plugin relies on
 	
 	:Key: The id of the dependent plugin
-	:Value: The version requirement the dependent plugin
+	:Value: The version requirement of the dependent plugin
 	"""
 
 	entrypoint: str
@@ -59,6 +65,8 @@ class Metadata:
 	archive_name: Optional[str]  # used in MCDR CLI only
 	resources: Optional[List[str]]  # used in MCDR CLI only
 
+	PLUGIN_ID_REGEX_OLD = re.compile(r'[a-z0-9_]{1,64}')
+	PLUGIN_ID_REGEX = re.compile(r'[a-z][a-z0-9_]{0,63}')
 	FALLBACK_VERSION = '0.0.0'
 
 	def __init__(self, data: Optional[dict], *, plugin: Optional['AbstractPlugin'] = None):
@@ -71,34 +79,38 @@ class Metadata:
 
 		def warn(*args, **kwargs):
 			if plugin is not None:
-				plugin.mcdr_server.logger.warning(*args, **kwargs)
+				plugin.mcdr_server.logger.warning(*args, **kwargs, stacklevel=3)
 
 		plugin_name_text = repr(plugin)
 
 		use_fallback_id_reason = None
-		self.id = data.get('id')
-		if self.id is None:
+		if (plugin_id := data.get('id')) is None:
 			use_fallback_id_reason = 'Plugin ID of {} not found'.format(plugin_name_text)
-		elif not isinstance(self.id, str) or re.fullmatch(r'[a-z0-9_]{1,64}', self.id) is None:
-			use_fallback_id_reason = 'Plugin ID "{}" of {} is invalid'.format(self.id, plugin_name_text)
+		else:
+			self.id = plugin_id
+			bad_id = not isinstance(self.id, str)
+			if self.PLUGIN_ID_REGEX.fullmatch(self.id) is None:
+				bad_id = True
+			if bad_id:
+				use_fallback_id_reason = 'Plugin ID {!r} of {} is invalid'.format(self.id, plugin_name_text)
 		if use_fallback_id_reason is not None:
 			if plugin is not None:
 				self.id = plugin.get_fallback_metadata_id()
 				warn('{}, use fallback id {} instead'.format(use_fallback_id_reason, self.id))
 			else:
 				raise ValueError('Plugin id not found in metadata')
-		class_util.check_type(self.id, str)
+		class_utils.check_type(self.id, str)
 
 		self.name = data.get('name', self.id)
 		if isinstance(self.name, RTextBase):
 			self.name = self.name.to_plain_text()
-		class_util.check_type(self.name, str)
+		class_utils.check_type(self.name, str)
 
 		description = data.get('description')
 		if isinstance(description, RTextBase):
 			description = description.to_plain_text()
 		self.description = description
-		class_util.check_type(self.description, [None, str, dict])
+		class_utils.check_type(self.description, (None, str, dict))
 
 		self.author = data.get('author')
 		if isinstance(self.author, str):
@@ -108,17 +120,17 @@ class Metadata:
 				self.author[i] = str(self.author[i])
 			if len(self.author) == 0:
 				self.author = None
-		class_util.check_type(self.author, [None, list])
+		class_utils.check_type(self.author, (None, list))
 
 		self.link = data.get('link')
-		class_util.check_type(self.link, [None, str])
+		class_utils.check_type(self.link, (None, str))
 
 		version_str = data.get('version')
 		if version_str:
 			try:
 				self.version = Version(version_str, allow_wildcard=False)
 			except VersionParsingError as e:
-				warn('Version "{}" of {} is invalid ({}), ignore and use fallback version instead {}'.format(version_str, plugin_name_text, e, self.FALLBACK_VERSION))
+				warn('Version {!r} of {} is invalid ({}), ignore and use fallback version instead {}'.format(version_str, plugin_name_text, e, self.FALLBACK_VERSION))
 				version_str = None
 		else:
 			warn("{} doesn't specific a version, use fallback version {}".format(plugin_name_text, self.FALLBACK_VERSION))
@@ -135,18 +147,18 @@ class Metadata:
 				))
 
 		self.entrypoint = data.get('entrypoint', self.id)
-		class_util.check_type(self.entrypoint, str)
+		class_utils.check_type(self.entrypoint, str)
 		# entrypoint module should be inside the plugin module
 		if self.entrypoint != self.id and not self.entrypoint.startswith(self.id + '.'):
-			raise ValueError('Invalid entry point "{}" for plugin id "{}"'.format(self.entrypoint, self.id))
+			raise ValueError('Invalid entry point {!r} for plugin id {!r}'.format(self.entrypoint, self.id))
 
 		self.archive_name = data.get('archive_name')
 		self.resources = data.get('resources', [])
-		class_util.check_type(self.archive_name, [None, str])
-		class_util.check_type(self.resources, list)
+		class_utils.check_type(self.archive_name, (None, str))
+		class_utils.check_type(self.resources, list)
 
 	def __repr__(self):
-		return class_util.represent(self)
+		return class_utils.represent(self)
 
 	def get_description(self, lang: Optional[str] = None) -> Optional[str]:
 		"""
@@ -155,11 +167,14 @@ class Metadata:
 		:param lang: Optional, the language to translate to. When not specified it will use the language of MCDR
 		:return: Translated plugin description
 		"""
+		if self.description is None:
+			return None
 		if isinstance(self.description, str):
 			return self.description
 		if lang is None:
-			lang = translation_util.get_mcdr_language()
-		return translation_util.translate_from_dict(self.description, lang, default=None)
+			lang = translation_utils.get_mcdr_language()
+		result = translation_utils.translate_from_dict(self.description, lang, default=None)
+		return class_utils.check_type(result, (str, None))
 
 	def get_description_rtext(self) -> RTextBase:
 		"""
@@ -167,10 +182,37 @@ class Metadata:
 
 		.. versionadded:: v2.1.2
 		"""
+		if self.description is None:
+			raise ValueError('description is None')
 		if isinstance(self.description, str):
 			return RText(self.description)
 		return RTextMCDRTranslation.from_translation_dict(self.description)
 
+	def to_dict(self) -> dict:
+		"""
+		Create a dict present of this metadata object
+
+		.. versionadded:: v2.13.0
+		"""
+		def copy(obj):
+			return obj.copy() if isinstance(obj, (list, dict)) else obj
+
+		return {
+			# Fields for all plugins
+			'id': self.id,
+			'version': str(self.version),
+			'name': self.name,
+			'description': copy(self.description),
+			'author': copy(self.author),
+			'link': self.link,
+			'dependencies': {k: str(v) for k, v in self.dependencies.items()},
+
+			# Fields for packed plugins
+			'entrypoint': self.entrypoint,
+			'archive_name': self.archive_name,
+			'resources': copy(self.resources),
+		}
+
 
 __SAMPLE_METADATA = {
 	'id': 'example_plugin',   # If missing it will be the file name without .py suffix
@@ -184,7 +226,7 @@ __SAMPLE_METADATA = {
 	'author': [
 		'Fallen_Breath'
 	],
-	'link': 'https://github.com/Fallen-Breath/MCDReforged',
+	'link': 'https://github.com/MCDReforged/MCDReforged',
 	'dependencies': {
 		'mcdreforged': '>=1.0.0'
 	},
@@ -197,3 +239,4 @@ __SAMPLE_METADATA = {
 		'another_resource_file',
 	]
 }
+Metadata(__SAMPLE_METADATA)  # there's should be no exception
